#include "Game.h"

u8 gameRunning = FALSE;

u8 levelStates[LEVEL_COUNT];

void drawGame()
{
    // Draw the content of the map on screen.
    drawMap();

    // Draw the units on the screen.
    drawUnits();

    // Draws the cursor on the screen if it is visible.
    drawCursor(xScroll, yScroll);

    // The hud will only update if hudRedraw is true (something has changed).
    drawHud();
}

void gameLogic()
{
    // Check if the player has completed the objectives for the level.
    if (checkObjectives())
    {
        // Extra money for winning the level (very generous).
        gameMoney += 800;
        levelEnded = LEVEL_END_COMPLETE;
    }

    // Turned off ship movement and doing cursor movement instead.
    moveCursor();

    // Runs through the player logic.
    playerLogic();

    // Move the player units.
    moveUnits();

    // Run the logic for the units deployed to the map.
    logicUnits();

    // Run the logic for the links on the map after the units have presumably moved.
    logicLinks();

    // Run the logic for the mines on the map.
    logicMines();

    // Check if the player has ended playing the level (either in victory or defeat).
    if (levelEnded)
    {
        // Set running to false to transition to the next level or the end of the game.
        gameRunning = FALSE;
    }
}

void initGame()
{
    gameRunning = TRUE;

    // Setup the map.
    initMap(currentLevel);

    // Initialise scroll values on game initialisation.
    xScroll = 0;
    yScroll = 0;

    // Setup the non player friendly ship.
    initShips();

    // Setup the hud.
    initHud();

    // Sets up the player specific functionality.
    initPlayer();

    // Initialise the sound effects.
    cpct_akp_musicInit(S_effects);
    cpct_akp_SFXInit(S_effects);
}

void readInput()
{
    // Reads the standard input controls.
    readGameControls();

    // Check that the exit key has been pressed.
    if (gameControls.controlFlags & CONTROL_EXIT_FLAG)
    {
        // If the player choses to press yes exit the game.
        if (exitDialog())
        {
            gameRunning = FALSE;
        }
        else
        {
            // Clear the screen before redrawing the map.
            cpct_clearScreen(0);

            // Force a map redraw to cleanup after the dialog is closed.
            mapRedraw = TRUE;

            // Make sure the units are redrawn after the dialog is closed.
            unitRedraw = TRUE;

            // Make sure the hud is redrawn after the dialog is closed.
            hudRedraw = TRUE;
        }
    }
}

void runGame()
{
    while (gameRunning)
    {
        // Read input from the player.
        readInput();

        // Run the game logic which will effect whats on the display.
        gameLogic();

        // Wait for vsync.
        cpct_waitVSYNC();

        // Draws the game on the screen.
        drawGame();
    }
}

void runGameSession()
{
    u8 i = 0;
    u8 levelsCompleted = 0;

    // Update the palette for either colour or green.
    updatePalette();

    // Reset game state values for a new game.
    resetGameSessionState();

    // First 3 levels are unlocked and ready by default.
    for (i = 0; i < 3; ++i)
    {
        levelStates[i] = LEVEL_READY;
    }

    // Last 6 levels will be unlocked by the player.
    for (i = 3; i < LEVEL_COUNT; ++i)
    {
        levelStates[i] = LEVEL_LOCKED;
    }

    do
    {
        // Runs the world\island map and displays it on screen.
        runMapScreen();

        if (levelEnded != GAME_OVER)
        {
            // Setup the level objectives.
            initObjectives();

            // Sets up the difficulty values based on the chosen difficulty.
            setupDifficulty();

            // Setup the game units.
            initUnits();

            // Displays help information for the chosen level;
            displayLevelHelp(currentLevel);

            // Displays the unit selection screen where the player can choose their units before starting a level.
            runUnitSelection();

            // Setup required before starting the game.
            initGame();

            // Clear the screen before starting the game.
            cpct_clearScreen(0);

            // Start the actual game.
            runGame();
        }

        // Update the level states if the player completed the level.
        if (levelEnded == LEVEL_END_COMPLETE)
        {
            levelStates[currentLevel] = levelEnded;

            if (levelsCompleted < LEVEL_COUNT - 3)
            {
                // Unlock each of the 6 locked levels on completing a level.
                levelStates[3 + levelsCompleted] = LEVEL_READY;
            }

            // Update the levels complete count;
            ++levelsCompleted;

            if (levelsCompleted == LEVEL_COUNT)
            {
                // All levels finished so game is complete.
                levelEnded = GAME_COMPLETE;
            }
        }
    }
    while (levelEnded < GAME_COMPLETE);
}