#include "Player.h"

void drawCursor(u8 xScroll, u8 yScroll)
{
    if (displayCursor)
    {
        u8* memory = cpct_getScreenPtr(CPCT_VMEM_START, (moveIndicator.x - xScroll) * TILE_WIDTH_BYTES, (moveIndicator.y - yScroll) * TILE_HEIGHT);

        // Normally shows the standard indicator which is currently white on all levels expect plant levels.
        cpct_drawSpriteMasked(G_cursor_white, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);

        // Check if the cursor is being used for adding a new unit to the map.
        if (selectedUnitSlot != 255 && units[selectedUnitSlot].deployed == FALSE)
        {
            // Check if the move indicator is outside of the zone.
            if (level_zone[currentLevel] == NULL ||
                (moveIndicator.x < level_zone[currentLevel][0] || moveIndicator.x > level_zone[currentLevel][2] ||
                    moveIndicator.y < level_zone[currentLevel][1] || moveIndicator.y > level_zone[currentLevel][3]))
            {
                // If the indicator is outside the zone display a red slash over the top of the cursor.
                cpct_drawSpriteMasked(G_unit_destroyed_x, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
            }
        }
    }    
}

void initPlayer()
{
    moveIndicator.x = 0;
    moveIndicator.y = 0;
}

void clearIndicator()
{
    u8 cell = MAPA(moveIndicator.x, moveIndicator.y);
    u16 mine = 0;

    // Draw over the old position for the move indicator.	
    drawCell(moveIndicator.x - xScroll, moveIndicator.y - yScroll, cell);

    // If the cell has a mine then we will need to make that is redrawn as well.
    if ((mine = findMine(moveIndicator.x, moveIndicator.y)))
    {
        u8* mapMemory = cpct_getScreenPtr(CPCT_VMEM_START, xPositions[moveIndicator.x - xScroll], yPositions[moveIndicator.y - yScroll]);
        drawMine(mapMines[mine], mapMemory);
    }
}

void moveCursor()
{
    // Only move the cursor when it is displayed.
    if (displayCursor)
    {
        // Sets the direction based on the user input.
        if (gameControls.controlFlags & CONTROL_UP_FLAG)
        {
            if (moveIndicator.y > 0)
            {
                clearIndicator();
                --moveIndicator.y;

                scrollScreenCursor(DIRECTION_UP);
            }
        }
        else if (gameControls.controlFlags & CONTROL_DOWN_FLAG)
        {
            if (moveIndicator.y + 1 < MAP_TILES_Y)
            {
                clearIndicator();
                ++moveIndicator.y;

                scrollScreenCursor(DIRECTION_DOWN);
            }
        }
        else if (gameControls.controlFlags & CONTROL_LEFT_FLAG)
        {
            if (moveIndicator.x > 0)
            {
                clearIndicator();
                --moveIndicator.x;

                scrollScreenCursor(DIRECTION_LEFT);
            }
        }
        else if (gameControls.controlFlags & CONTROL_RIGHT_FLAG)
        {
            if (moveIndicator.x + 1 < MAP_TILES_X)
            {
                clearIndicator();
                ++moveIndicator.x;

                scrollScreenCursor(DIRECTION_RIGHT);
            }
        }
    }

    // Ensures the speed for movement is right especially with WinCPCtelera.
    if (gameControls.controlFlags)
    {
#ifdef WINCPCTELERA
        // Doing a wait provides a workable equivalent to the CPC version below.
        wincpct_wait(400);
#else
        // Wait for any previous key presses to clear first so that the cursor only moves one cell at at time.
        while (cpct_isAnyKeyPressed());
#endif
    }
}

void playerLogic()
{
    u8 i = 0;
    u8 flag = 1;

    // If the select unit is dead then reset the selected unit to 255.
    if (selectedUnitSlot != 255 && units[selectedUnitSlot].dead)
    {
        selectedUnitSlot = 255;

        hudRedraw = TRUE;
    }

    if (gameControls.controlFlags2 & CONTROL_END_TURN_FLAG)
    {
        // Reset flag to 0 indicating all player moves this turn are done.
        playerUnitTurnFlag = 0;

        // Switch turn state to friendly ships.
        gameTurnState = FRIENDLY_SHIP_TURN;

        if (units[selectedUnitSlot].target.x != 255 && units[selectedUnitSlot].target.y != 255)
        {
            // Removed direction when ending the turn to make sure that on the next turn for that unit
            // that it doesn't run through the unit logic when it shouldn't.
            units[selectedUnitSlot].direction = 0;
        }

#ifdef WINCPCTELERA
        // Doing a wait provides a workable equivalent to the CPC version below.
        wincpct_wait(400);
#else
        // Wait for any previous key presses to clear first so that you can't hold down the end turn button.
        while (cpct_isAnyKeyPressed());
#endif
    }

    if (gameTurnState == PRE_PLAYER_TURN)
    {
        for (i = 0; i < MAX_PLAYER_UNIT_COUNT; ++i)
        {
            if (units[i].deployed)
            {
                playerUnitTurnFlag |= (1 << i);
            }
        }

        // Subtract from remaining turns every time the player has completed their turn.
        --remainingTurns;
        
        unitRedraw = TRUE;

        // Redraw the hud following updating the unit turn flag.
        hudRedraw = TRUE;

        gameTurnState = PLAYER_TURN;
    }

    // Selects a unit based on the unit number pressed.
    for (i = 0; i < MAX_PLAYER_UNIT_COUNT; ++i)
    {
        if (gameControls.unitNumberFlags & flag)
        {
            // Check that there is a valid unit in the slot that is alive.
            if (units[i].type != 255 && !units[i].dead)
            {
                // Updates the selected unit slot to match the current slot.
                selectedUnitSlot = i;

                // Clear active unit as a different unit is selected.
                activeUnit = 255;

                // Clear where the indicator is currently.
                clearIndicator();

                // Update the indicator to the position of the selected unit if it has already been deployed.
                if (units[i].deployed)
                {
                    moveIndicator.x = units[i].position.x;
                    moveIndicator.y = units[i].position.y;
                }
                else
                {
                    // Default the indicator to the start of the level zone.
                    moveIndicator.x = level_zone[currentLevel][0];
                    moveIndicator.y = level_zone[currentLevel][1];
                }

                // Move the screen to the position of the indicator.
                moveScreen(moveIndicator.x, moveIndicator.y);

                // Redraws the hud to show the selected unit indicator.
                hudRedraw = TRUE;

                // Makes the cursor visible once a unit has been selected.
                displayCursor = TRUE;
                break;
            }
        }

        // Move to the next flag value for the next unit number.
        flag <<= 1;
    }

    if (gameControls.controlFlags & CONTROL_USE_FLAG)
    {
        // Check that a unit slot is currently selected.
        if (selectedUnitSlot != 255)
        {
            Unit* selectedUnit = units + selectedUnitSlot;

            // Check that the unit has not previously been deployed and is not dead.
            if (selectedUnit->deployed == FALSE && selectedUnit->dead == FALSE)
            {
                // Only let a unit spawn in the starting zone of the level if there is one.
                if (level_zone[currentLevel] == NULL ||
                    (moveIndicator.x >= level_zone[currentLevel][0] && moveIndicator.x <= level_zone[currentLevel][2] &&
                     moveIndicator.y >= level_zone[currentLevel][1] && moveIndicator.y <= level_zone[currentLevel][3]))
                {
                    // Add a unit to the units array.
                    selectedUnit->position.x = moveIndicator.x;
                    selectedUnit->position.y = moveIndicator.y;
                    selectedUnit->width = unit_width[selectedUnit->type];
                    selectedUnit->height = unit_height[selectedUnit->type];

                    // If we place this here we can pass the current unit object
                    // but throw it away without activating the object on the map if the next position is solid.
                    if (unitNextPositionIsSolid(selectedUnit, FALSE))
                    {
                        // Looks like there is something blocking the unit being spawned so returning.
                        return;
                    }

                    selectedUnit->deployed = TRUE;
                    selectedUnit->health = 10;
                    selectedUnit->unitClass = PLAYER_UNIT_CLASS;
                    selectedUnit->defusing = FALSE;
                    selectedUnit->dead = FALSE;

                    // There can only be one escort ship and it must be a transport ship.
                    if (escortShipIndex == 255 && selectedUnit->type == UNIT_TYPE_TRANSPORT)
                    {
                        escortShipIndex = selectedUnitSlot;
                    }

                    // Convert selectUnitSlot to a flag and mark the unit as ready to have its turn.
                    playerUnitTurnFlag |= (1 << selectedUnitSlot);

                    // Redraw the hud to update the unit action on the hud.
                    hudRedraw = TRUE;

                    // Enable unit redraw when adding a unit.
                    unitRedraw = TRUE;
                }
            }
            else
            {
                // Activate the unit so that we can run the unit logic.
                if (activeUnit == selectedUnitSlot)
                {
                    // Deactivate the unit.
                    activeUnit = 255;

                    // Turn on the display cursor.
                    displayCursor = TRUE;
                }
                else
                {
                    activeUnit = selectedUnitSlot;

                    // Remove indicator.
                    clearIndicator();

                    // Turn off the display cursor.
                    displayCursor = FALSE;

                    moveScreen(selectedUnit->position.x, selectedUnit->position.y);

                    // Reset unit next position values on unit activation.
                    selectedUnit->direction = 0;
                    selectedUnit->nextPosition.x = selectedUnit->position.x;
                    selectedUnit->nextPosition.y = selectedUnit->position.y;
                }
            }
        }
    }
    else if (gameControls.controlFlags & CONTROL_TARGET_FLAG)
    {
        // Check that a unit slot is currently selected.
        if (selectedUnitSlot != 255)
        {
            // Check that the unit has previously been deployed.
            if (units[selectedUnitSlot].deployed)
            {
                // Sets the target for the selected unit to the indicator position.
                units[selectedUnitSlot].target.x = moveIndicator.x;
                units[selectedUnitSlot].target.y = moveIndicator.y;

                // Hide the cursor as it is no longer required.
                displayCursor = FALSE;
            }
        }
    }

    // Puts the unit in defuse mode if the unit is an active diver and the defuse key is pressed.
    if (gameControls.controlFlags & CONTROL_DEFUSE_FLAG)
    {
        if (activeUnit != 255)
        {
            if (units[activeUnit].type == UNIT_TYPE_DIVER)
            {
                // 10 is the expected number to start the defusing sequence at.
                units[activeUnit].defusing = 10;
            }
            else if (units[activeUnit].type == UNIT_TYPE_RADAR)
            {
                // Only start the radar scan if one is not already in progress.
                if (!units[activeUnit].defusing)
                {
                    units[activeUnit].defusing = 10;
                }
            }
            else if (units[activeUnit].type == UNIT_TYPE_MAGNET)
            {
                // 3 is the expected number to start the defusing sequence at.
                units[activeUnit].defusing = 3;
            }
            else if (units[activeUnit].type == UNIT_TYPE_EXPLOSIVE_DIVER)
            {
                units[activeUnit].defusing = 2;
            }
            else if (units[activeUnit].type == UNIT_TYPE_TRANSPORT)
            {
                // Reset the first warship target to the current position of the transport ship.
                if (activeUnit == escortShipIndex)
                {
                    u8 i = 0;

                    for (i = 0; i < friendlyShipCount; ++i)
                    {
                        u8 warshipIndex = FRIENDLY_SHIP_UNIT_START + i;

                        if (units[warshipIndex].type == UNIT_TYPE_WARSHIP)
                        {
                            // Taking into account the width of the warship when choosing the new target.
                            units[warshipIndex].target.x = units[activeUnit].position.x - units[warshipIndex].width;
                            units[warshipIndex].target.y = units[activeUnit].position.y;
                            break;
                        }
                    }
                }
            }
            else if (units[activeUnit].type == UNIT_TYPE_KAMAKAZI)
            {
                units[activeUnit].defusing = 2;
            }
        }
    }
}