#include "Unit.h"

u8 playerUnitCount = 0;
u8 friendlyShipCount = 0;
Unit units[MAX_UNIT_COUNT];
u8 playerUnitTurnFlag = 0;

u8 containsUnit(u8 x, u8 y, Unit* unit)
{
    u8 i = 0;

    for (i = 0; i < MAX_UNIT_COUNT; ++i)
    {
        // Skip a unit if it has no health (the ship is dead or not deployed).
        if (units[i].health != 0)
        {
            // Skip a unit if it is the same as the current unit.
            if (unit != NULL && units[i].id == unit->id)
            {
                continue;
            }

            if (x >= units[i].position.x && x < units[i].position.x + units[i].width)
            {
                if (y >= units[i].position.y && y < units[i].position.y + units[i].height)
                {
                    // Adding one so we can get the index at the caller end by subtracting one.
                    return i + 1;
                }
            }
        }
    }

    return FALSE;
}

void deliverKamikaziPayload(Unit* unit)
{
    if (unit->type == UNIT_TYPE_KAMAKAZI)
    {
        u8 i = 0;

        // All mines in the radius will be updated.
        for (i = 0; i < MINE_TYPE_COUNT; ++i)
        {
            updateMinesAroundUnit(unit->position.x, unit->position.y, unit->width, unit->height, 3, i, MINE_EXPLOSION_GRAPHIC_START);
        }
    }
}

void drawUnits()
{
    if (unitRedraw)
    {
        u8 i = 0;
        u8 j = 0;

        for (i = 0; i < MAX_UNIT_COUNT; ++i)
        {
            // Only draw units that have health.
            if (units[i].health)
            {
                // Using signed numbers to work out the width for a partial draw of the unit.
                i8 x = units[i].position.x - xScroll;
                i8 y = units[i].position.y - yScroll;

                i8 width = units[i].width;
                i8 height = units[i].height;

                // Calculates the width and height if only part of the unit is on screen.
                if (x < 0)
                {
                    width += x;
                }

                if (y < 0)
                {
                    height += y;
                }

                // Make sure the unit is not completely off the screen (width\height is 0 or less)
                if (width > 0 && height > 0)
                {
                    // Drawing all cells of the unit.
                    for (j = 0; j < units[i].width * units[i].height; ++j)
                    {
                        // Check x and y for each cell in the unit that they are on the screen.
                        if (x >= 0 && x < SCREEN_TILES_X)
                        {
                            if (y >= 0 && y < SCREEN_TILES_Y)
                            {
                                // The memory location of the current cell of the unit.
                                u8* memory = cpct_getScreenPtr(CPCT_VMEM_START, x * TILE_WIDTH_BYTES, y * TILE_HEIGHT);

                                // Draws an individual cell of the unit.
                                if (i < FRIENDLY_SHIP_UNIT_START)
                                {
                                    // Player units 0 - 7.
                                    cpct_drawSprite(unitGraphics[units[i].graphicIndex + j], memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
                                }
                                else
                                {
                                    // Non player ships 8 - 11.
                                    cpct_drawSprite(shipGraphics[units[i].type + j], memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
                                }
                            }
                        }

                        if ((j + 1) % units[i].width == 0)
                        {
                            // Move y to the next line.
                            ++y;

                            // Reset x for the next line.
                            x -= (units[i].width - 1);
                        }
                        else
                        {
                            // Move to the cell in the row of the unit.
                            ++x;
                        }
                    }
                }
            }
        }
    
        unitRedraw = FALSE;
    }
}

void initUnits()
{
    u8 i = 0;
    Unit* unit = units;

    // Initialise counts to 0.
    playerUnitCount = 0;
    friendlyShipCount = 0;

    // Initialise all units to have a health of 0 so that they will not be drawn until they have health.
    for (i = 0; i < MAX_UNIT_COUNT; ++i)
    {
        unit->health = 0;

        unit->dead = FALSE;

        unit->deployed = FALSE;

        // Using 255 to indicate no target has been set.
        unit->target.x = 255;
        unit->target.y = 255;

        // Using 255 to indicate there is no unit set.
        unit->type = 255;

        // Set the id to be the same as the array index.
        unit->id = i;

        // Initialise target index to 255 to indicate not set.
        unit->targetStartIndex = 255;
        unit->targetEndIndex = 255;

        // Move units pointer to the next unit in the array.
        ++unit;
    }

    // Mark all player units as not their turn (0).
    playerUnitTurnFlag = 0;
}

void logicUnits()
{
    u8 i = 0;

    // Only run player logic when it is the player's units turn.
    if (gameTurnState == PLAYER_TURN)
    {
        // If the player is moving the cursor the logic does not need to be run until
        // they have completed their chosen action.
        if (!displayCursor)
        {
            for (i = 0; i < MAX_PLAYER_UNIT_COUNT; ++i)
            {
                // Only run the logic if the current unit is the active unit or no units are active.
                if (activeUnit == 255 || activeUnit == i || (units[i].target.x != 255 && units[i].target.y != 255) || units[i].defusing)
                {
                    // Only run the logic for the units that have been deployed to the map and whoes turn is current.
                    if (units[i].deployed && (playerUnitTurnFlag & (1 << i)))
                    {
                        // Make the unit invulnerable to moving mines if it is defusing.
                        if (!units[i].defusing)
                        {
                            // Resets the unit health to 0 as a mine is at the unit's position.
                            unitNextPositionIsSolid(&units[i], FALSE);
                        }

                        // Check to see if the unit is alive
                        if (units[i].health == 0)
                        {
                            // Start the death delay
                            deathDelay = TRUE;

                            // Reset the timer.
                            timerCount = 0;

                            // Play the explosion sound effect.
                            playSound(SOUND_EXPLOSION);

                            // Set deployed to FALSE to indicate the unit is no longer deployed to the map.
                            units[i].deployed = FALSE;

                            // Marked the unit as dead.
                            units[i].dead = TRUE;

                            // Redraw the hud to show unit dead on hud.
                            hudRedraw = TRUE;

                            // Mark unit turn flag as complete after the unit is dead.
                            playerUnitTurnFlag &= ~(1 << i);

                            // Move to the next unit.
                            continue;
                        }

                        // Cabled units are always defusing as they move.
                        if (units[i].type == UNIT_TYPE_CABLED || units[i].type == UNIT_TYPE_CABLED_PAIR)
                        {
                            units[i].defusing = TRUE;
                        }

                        // Only run logic if unit is active, defusing or moving to a target.
                        if (activeUnit == i || units[i].defusing || (units[i].target.x != 255 && units[i].target.y != 255))
                        {
                            // Moves the unit to the next position or target
                            // Only process the cell if the unit currently has a direction (is moving).
                            logicUnitToNextPosition(&units[i], i);

                            // If the unit is defusing attempt to defuse a mine.
                            if (units[i].defusing)
                            {
                                int x = units[i].position.x;
                                int y = units[i].position.y;

                                if (units[i].type == UNIT_TYPE_DIVER)
                                {
                                    u8 j = 0;

                                    if (units[i].defusing == 10)
                                    {
                                        for (j = 0; j < 9; ++j)
                                        {
                                            u16 mine = findMine(x + xAroundCellPositions[j], y + yAroundCellPositions[j]);

                                            if (mine)
                                            {
                                                // Move the unit on top of the mine.
                                                units[i].position.x = x + xAroundCellPositions[j];
                                                units[i].position.y = y + yAroundCellPositions[j];

                                                // Update screen position to the new position of the unit.
                                                moveScreen(units[i].position.x, units[i].position.y);

                                                // The defusing number counts down the number of turns to wait before the defuse is finished.
                                                units[i].defusing = 5;

                                                // Having moved onto a mine we don't need to find any more mines.
                                                break;
                                            }
                                        }

                                        if (units[i].defusing == 10)
                                        {
                                            // No mines found so end defuse.
                                            units[i].defusing = 0;
                                        }
                                    }
                                    else
                                    {
                                        --units[i].defusing;

                                        // Defuse the mine after wait is over.
                                        if (!units[i].defusing)
                                        {
                                            u16 mineIndex = findMine(x, y);

                                            // Reset mine type to 255 as it has been defused.
                                            mapMines[mineIndex] = 255;

                                            // Update the mine count following defuse.
                                            --mineCount;

                                            // Add 100 money for defusing the mine.
                                            gameMoney += 100;

                                            // Redraw hud following money increase.
                                            hudRedraw = TRUE;
                                        }
                                    }
                                }
                                else if (units[i].type == UNIT_TYPE_MAGNET)
                                {
                                    if (units[i].defusing == 3)
                                    {
                                        // Set any bottom mines to 255 (invalid) as they will be then be considered defused.
                                        u8 minesDefused = updateMinesAroundUnit(x, y, units[i].width, units[i].height, 1, MINE_TYPE_BOTTOM_0, 255);
                                        minesDefused += updateMinesAroundUnit(x, y, units[i].width, units[i].height, 1, MINE_TYPE_BOTTOM_REVEALED_GRAPHIC, 255);

                                        // Add to game money for every mine defused.
                                        gameMoney += minesDefused * MAGNET_MINE_SALVAGE_MONEY_ADD;

                                        // Update mine count by the number of mines defused by the magnet minesweeper.
                                        mineCount -= minesDefused;
                                    }

                                    // Magnetic defuse is complete so reduce defusing value.
                                    --units[i].defusing;

                                    // Redraw the hud to show hud action for magnet defuse.
                                    hudRedraw = TRUE;

                                    // Redraw the units as the map is being redrawn.
                                    unitRedraw = TRUE;

                                    // Redraw the map to draw the revealed mines.
                                    mapRedraw = TRUE;
                                }
                                else if (units[i].type == UNIT_TYPE_RADAR)
                                {
                                    if (units[i].defusing == 10)
                                    {
                                        // Reveal any bottom mines within the radius of the radar ship.
                                        updateMinesAroundUnit(x, y, units[i].width, units[i].height, 3, MINE_TYPE_BOTTOM_0, MINE_TYPE_BOTTOM_REVEALED_GRAPHIC);
                                    }
                                    else if (units[i].defusing == 8)
                                    {
                                        // Restore bottom mines to be being invisible.
                                        updateMinesAroundUnit(x, y, units[i].width, units[i].height, 3, MINE_TYPE_BOTTOM_REVEALED_GRAPHIC, MINE_TYPE_BOTTOM_0);
                                    }

                                    // Radar scan is complete so reduce defusing value.
                                    // Radar cannot rescan until defusing returns to 0.
                                    --units[i].defusing;

                                    // Redraw the hud to show hud action for radar scan.
                                    hudRedraw = TRUE;

                                    // Redraw the units as the map is being redrawn.
                                    unitRedraw = TRUE;

                                    // Redraw the map to draw the revealed mines.
                                    mapRedraw = TRUE;
                                }
                                else if (units[i].type == UNIT_TYPE_CABLED || units[i].type == UNIT_TYPE_CABLED_PAIR)
                                {
                                    u8 j = 0;
                                    u8 k = 0;

                                    // Defuse any mines underneath the unit.
                                    for (j = x; j < units[i].width + x; ++j)
                                    {
                                        for (k = y; k < units[i].height + y; ++k)
                                        {
                                            u16 mine = findMine(j, k);

                                            if (mine)
                                            {
                                                if (mapMines[mine] == MINE_TYPE_BOTTOM_0 || mapMines[mine] == MINE_TYPE_BOTTOM_REVEALED_GRAPHIC)
                                                {
                                                    mapMines[mine] = 255;

                                                    // Reduce mine count for mines defused.
                                                    --mineCount;

                                                    // Update the hud in case any mine have been defused.
                                                    // It must be done here otherwise we can end up with a flicker.
                                                    hudRedraw = TRUE;
                                                }
                                            }
                                        }
                                    }

                                    units[i].defusing = 0;
                                }
                                else if (units[i].type == UNIT_TYPE_EXPLOSIVE_DIVER)
                                {
                                    if (units[i].defusing == 2)
                                    {
                                        u8 j = 0;

                                        for (j = 0; j < 9; ++j)
                                        {
                                            u16 mine = findMine(x + xAroundCellPositions[j], y + yAroundCellPositions[j]);

                                            if (mine)
                                            {
                                                // Can only set explosive mines on moored or floating mines but not bottom mines.
                                                if (mapMines[mine] >= MINE_TYPE_MOOR_0 && mapMines[mine] <= MINE_TYPE_FLOAT_1)
                                                {
                                                    // Had planned to move the player on top of the mine but decided it was easier not to do that.
                                                    // Update the mine so that it is ready to explode but give time for the diver to run.
                                                    mapMines[mine] = MINE_EXPLOSIVE_SET_GRAPHIC_START;

                                                    // There will be a little bit of cool down before the player can move again.
                                                    units[i].defusing = 1;

                                                    // Redraw the hud to show hud action.
                                                    hudRedraw = TRUE;

                                                    // Redraw the units as the map is being redrawn.
                                                    unitRedraw = TRUE;

                                                    // Redraw the map to draw the exlosive charge on the mine.
                                                    mapRedraw = TRUE;

                                                    // Having moved onto a mine we don't need to find any more mines.
                                                    break;
                                                }
                                            }
                                        }

                                        if (units[i].defusing == 2)
                                        {
                                            // No mines found so end defuse.
                                            units[i].defusing = 0;
                                        }
                                    }
                                    else
                                    {
                                        // Remove normal movement, mostly because I wanted to use the hud action for explosive.
                                        units[i].defusing = 0;
                                    }
                                }
                                else if (units[i].type == UNIT_TYPE_KAMAKAZI)
                                {
                                    // Kill the unit on detonating the payload.
                                    units[i].health = 0;

                                    // If the unit that died was a kamakazi vescel then make a bigger boom!
                                    deliverKamikaziPayload(&units[i]);

                                    // Reset defusing.
                                    units[i].defusing = 0;
                                }

                                // Turn flag does not apply to cabled unit otherwise turns don't work correctly for it.
                                if (units[i].type != UNIT_TYPE_CABLED && units[i].type != UNIT_TYPE_CABLED_PAIR)
                                {
                                    // Convert i to a flag and mark the unit doing the defusing has completed their turn.
                                    playerUnitTurnFlag &= ~(1 << i);
                                }
                            }
                        }

                        // Only change game turn state after the player has completed their moves.
                        if (gameTurnState == PLAYER_TURN && !playerUnitTurnFlag)
                        {
                            gameTurnState = FRIENDLY_SHIP_TURN;
                        }
                    }
                }
            }
        }
    }

    if (gameTurnState == FRIENDLY_SHIP_TURN)
    {
        // Point unit to the first friendly ship in the units array.
        Unit* unit = units + FRIENDLY_SHIP_UNIT_START;

        for (i = FRIENDLY_SHIP_UNIT_START; i < FRIENDLY_SHIP_UNIT_START + friendlyShipCount; ++i)
        {
            // Make sure the unit is alive.
            if (unit->health > 0)
            {
                if (unit->target.x == 255 && unit->target.y == 255)
                {
                    // Random movement if there is no target (or has reached the target).
                    u8 j = 0;
                    u8 x = unit->position.x;
                    u8 y = unit->position.y;

                    // Resets the unit health to 0 as a mine is at the unit's position.
                    if (unitNextPositionIsSolid(&units[i], FALSE))
                    {
                        if (unit->health == 0)
                        {
                            // If the unit now has no health then a mine is at the unit so we don't need to move the unit.
                            continue;
                        }
                    }

                    // Determine possible directions so if one is blocked the mine can move a different way.
                    determineRandomDirections();

                    // Iterate through the random directions so that if it is blocked on one it will chose another.
                    for (j = 0; j < DIRECTION_COUNT; ++j)
                    {
                        u8 direction = determinedDirections[j];

                        // Determines the next position for the mine based on the determined random direction.
                        unit->nextPosition.x = x + xDirectionChange[direction];
                        unit->nextPosition.y = y + yDirectionChange[direction];

                        // Make sure next x,y is not outside the range of the map.
                        if (unit->nextPosition.x >= MAP_TILES_X)
                        {
                            // Blocked on x axis.
                            continue;
                        }
                        else if (unit->nextPosition.y >= MAP_TILES_Y)
                        {
                            // Blocked on y axis.
                            continue;
                        }
                        else if (unitNextPositionIsSolid(unit, TRUE))
                        {
                            // The cell is blocked by a tile.
                            continue;
                        }

                        // Update the unit position to the newly determined (random) position.
                        unit->position.x = unit->nextPosition.x;
                        unit->position.y = unit->nextPosition.y;

                        // Unit moved so redraw it.
                        unitRedraw = TRUE;

                        // Successfully updated the friendly ship position so break out of the direction loop.
                        break;
                    }
                }
                else
                {
                    // Moving the friendly ship to the target x,y.
                    // Only process the cell if the unit currently has a direction (is moving).
                    logicUnitToNextPosition(unit, i);
                }
            }

            // Move to the next element in the units array.
            ++unit;
        }

        // Redraw the map after ship positions have changed.
        mapRedraw = TRUE;

        // Change to mine turn following friendly ship turn.
        gameTurnState = MINE_TURN;
    }
}

void logicUnitToNextPosition(Unit* unit, u8 unitIndex)
{
    // Only process the cell if the unit currently has a direction (is moving).
    if (unit->direction)
    {
        // Checks if every cell that the unit is on (next position) is solid or not.
        u8 futureCell = unitNextPositionIsSolid(unit, TRUE);

        // Let the unit move over the top of an empty cell, player unit can move over item.
        if (!futureCell || (unit->unitClass == PLAYER_UNIT_CLASS && futureCell & CELL_ITEM_FLAG))
        {
            // Redraw the cell at the unit previous position.
            u8 drawX = unit->position.x - xScroll;
            u8 drawY = unit->position.y - yScroll;

            // Only needed if it is on screen.
            if (drawX < SCREEN_TILES_X && drawY < SCREEN_TILES_Y)
            {
                u8 previousCell = MAPA(unit->position.x, unit->position.y);

                drawCell(unit->position.x - xScroll, unit->position.y - yScroll, previousCell);
            }

            // Update unit position.
            unit->position.x = unit->nextPosition.x;
            unit->position.y = unit->nextPosition.y;
            
            if (unit->unitClass == PLAYER_UNIT_CLASS) 
            {
                // Scroll the screen when the active player unit has moved.
                if (activeUnit == unitIndex)
                {
                    moveScreen(unit->position.x, unit->position.y);
                }

                // Update the move indicator to follow the unit move.
                moveIndicator.x = units[unitIndex].nextPosition.x;
                moveIndicator.y = units[unitIndex].nextPosition.y;
                
                // Redraw the hud to show any changes due to unit move.
                hudRedraw = TRUE;

                // Convert i to a flag and mark the player unit as having completed their turn.
                playerUnitTurnFlag &= ~(1 << unitIndex);
            }

            // Redraw the map to cover up the position that the unit was on the screen.
            mapRedraw = TRUE;

            // Unit has moved so redraw units on screen.
            unitRedraw = TRUE;
        }
        else
        {
            // Unit is blocked so reset the target if it is set.
            // The reason for doing this is that if we let the player move around something and then continue to the
            // target it feels like they do not have complete control of where their units are moving to.
            if (unit->target.x != 255 && unit->target.y != 255)
            {
                // We don't want non-player units resetting their target when they get blocked.
                // This is essential for gated trade lane.
                if (unit->unitClass == PLAYER_UNIT_CLASS)
                {
                    unit->target.x = unit->position.x;
                    unit->target.y = unit->position.y;
                }
            }
        }

        // Reset movement.
        unit->direction = 0;
    }
}

void moveUnits()
{
    u8 i = 0;
    Unit* unit = units;

    for (i = 0; i < MAX_UNIT_COUNT; ++i)
    {
        // Move if a player unit is active or any unit that has a target (includes friendly).
        if (i == activeUnit || (unit->target.x != 255 && unit->target.y != 255))
        {
            // Unit cannot move while it is defusing.
            if (!unit->defusing)
            {
                i8 moveX = 0;
                i8 moveY = 0;

                // Check if a target has been set.
                if (unit->target.x != 255 && unit->target.y != 255)
                {
                    // Check that the target has not been reached.
                    if (unit->target.x != unit->position.x || unit->target.y != unit->position.y)
                    {
                        if (unit->target.x > unit->position.x)
                        {
                            unit->direction = DIRECTION_RIGHT;
                            moveX = 1;
                        }
                        else if (unit->target.x < unit->position.x)
                        {
                            unit->direction = DIRECTION_LEFT;
                            moveX = -1;
                        }
                        else if (unit->target.y > unit->position.y)
                        {
                            unit->direction = DIRECTION_DOWN;
                            moveY = 1;
                        }
                        else if (unit->target.y < unit->position.y)
                        {
                            unit->direction = DIRECTION_UP;
                            moveY = -1;
                        }
                    }
                    else
                    {
                        // Only need to support daisy chaining of targets for ship class units.
                        if (unit->unitClass == SHIP_UNIT_CLASS)
                        {
                            if (escortShipIndex == 255)
                            {
                                // Move unit along target array (x and y).
                                unit->targetStartIndex += 2;

                                // Check to see if the unit has moved to the end of the target or not.
                                if (unit->targetStartIndex != unit->targetEndIndex)
                                {
                                    unit->target.x = currentLevelTargets[unit->targetStartIndex];
                                    unit->target.y = currentLevelTargets[unit->targetStartIndex + 1];
                                }
                                else
                                {
                                    // When the units have reached their target reset the target to 255 (unset equivalent).
                                    unit->target.x = 255;
                                    unit->target.y = 255;

                                    // Unit has reached the end of the target which means it should have reached the objective.
                                    updateObjectives();
                                }
                            }
                            else
                            {
                                // Updates the target to follow the escort ship.
                                if (unit->type == UNIT_TYPE_WARSHIP)
                                {
                                    unit->target.x = units[escortShipIndex].position.x - unit->width;
                                    unit->target.y = units[escortShipIndex].position.y;
                                }
                            }
                        }
                        else
                        {
                            // When the units have reached their target reset the target to 255 (unset equivalent).
                            unit->target.x = 255;
                            unit->target.y = 255;
                        }
                    }
                }
                else
                {
                    // Game controls are not relevent when moving to target.
                    if (gameControls.controlFlags & CONTROL_UP_FLAG)
                    {
                        unit->direction = DIRECTION_UP;
                        moveY = -1;
                    }
                    else if (gameControls.controlFlags & CONTROL_RIGHT_FLAG)
                    {
                        unit->direction = DIRECTION_RIGHT;
                        moveX = 1;
                    }
                    else if (gameControls.controlFlags & CONTROL_DOWN_FLAG)
                    {
                        unit->direction = DIRECTION_DOWN;
                        moveY = 1;
                    }
                    else if (gameControls.controlFlags & CONTROL_LEFT_FLAG)
                    {
                        unit->direction = DIRECTION_LEFT;
                        moveX = -1;
                    }
                }

                // Determine where the unit can go next based on direction.
                if (unit->direction)
                {
                    // Set potential next position based on move values.
                    unit->nextPosition.x = unit->position.x + moveX;
                    unit->nextPosition.y = unit->position.y + moveY;

                    // Move left (255 is effectively -1).
                    if (unit->nextPosition.x == 255)
                    {
                        // Left boundary so reset to current position.
                        unit->nextPosition.x = unit->position.x;
                    }

                    // Move right.
                    if (unit->nextPosition.x + unit->width > MAP_TILES_X)
                    {
                        // Right boundary so reset to current position.
                        unit->nextPosition.x = unit->position.x;
                    }

                    // Move up (255 is effectively -1).
                    if (unit->nextPosition.y == 255)
                    {
                        // Top boundary so reset to current position.
                        unit->nextPosition.y = unit->position.y;
                    }

                    // Move down.
                    if (unit->nextPosition.y + unit->height > MAP_TILES_Y)
                    {
                        // Bottom boundary so reset to current position.
                        unit->nextPosition.y = unit->position.y;
                    }
                }
            }
        }
    
        // Move unit to next unit in the array.
        ++unit;
    }
}

u8 processCellUnit(Unit* unit, u8 x, u8 y)
{
    u8 cell = MAPA(x, y);
    u16 mineCell = findMine(x, y);
    u8 unitIndex = 0;

    if (cell & CELL_ITEM_FLAG)
    {
        // Clear the cell after empty the item is "picked up".
        MAPA(x, y) = CELL_EMPTY;

        // TODO pick up the item and load it into the player inventory.

        // Draw over the cell with a blank rectangle.
        drawBlankCellScroll(x, y);

        // If the cell is empty return 0.
        return 0;
    }
    else if (mineCell)
    {
        if (unit->type == UNIT_TYPE_CABLED || unit->type == UNIT_TYPE_CABLED_PAIR)
        {
            if (mapMines[mineCell] == MINE_TYPE_BOTTOM_0 || mapMines[mineCell] == MINE_TYPE_BOTTOM_REVEALED_GRAPHIC)
            {
                // Cabled ships have immunity to bottom mines so can walk over them without dieing.
                return 0;
            }
        }
        
        // Marine life will not run into mines.
        if (unit->type != UNIT_TYPE_TURTLE)
        {
            // Change the mine to the exploded mine graphic.
            mapMines[mineCell] = MINE_EXPLOSION_GRAPHIC_START;

            // Set unit health to 0 on mine contact.
            unit->health = 0;

            if (unit->carryingCargo)
            {
                // Instant fail of objectives if the ship carrying the cargo gets destroyed.
                failObjectives();
            }

            // Check if the unit was a non player ship.
            if (unit->unitClass == SHIP_UNIT_CLASS)
            {
                // Update the friendly ship count on unit death.
                --friendlyShipCount;
            }

            // If the unit that died was a kamakazi vescel then make a bigger boom!
            deliverKamikaziPayload(unit);

            // Redraw units on mine explosion.
            unitRedraw = TRUE;

            // Force a map redraw of the map to show the exploded mine graphic.
            mapRedraw = TRUE;
        }

        // Mine cell is returned as solid so the player can see explode graphic in full.
        return 1;
    }
    else if (cell & CELL_INTERACTABLE_FLAG)
    {
        if (unit->unitClass != SHIP_UNIT_CLASS)
        {
            if ((cell & CELL_GFX_RANGE) == INTERACTABLE_TYPE_PRESSURE_PLATE)
            {
                // The pressure plate is considered not solid.
                return 0;
            }
            else if ((cell & CELL_GFX_RANGE) == INTERACTABLE_TYPE_PLATFORM_FULL || (cell & CELL_GFX_RANGE) == INTERACTABLE_TYPE_PLATFORM_DESTINATION)
            {
                // Only transport ships can move over platforms.
                if (unit->type == UNIT_TYPE_TRANSPORT)
                {
                    // Check if the warship is near and complete if all good.
                    if (currentLevel == LEVEL_ESCORT)
                    {
                        // Only unit 8 is the warship on LEVEL_ESCORT so we can get away with using the FRIENDLY_SHIP_UNIT_START as the index.
                        if (units[FRIENDLY_SHIP_UNIT_START].position.x + units[FRIENDLY_SHIP_UNIT_START].width + 4 > unit->position.x &&
                            units[FRIENDLY_SHIP_UNIT_START].position.y + 1 + 3 > unit->position.y)
                        {
                            updateObjectives();
                        }
                    }

                    // A full platform or a destination is treated as not solid.
                    return 0;
                }
            }

            return 1;
        }
        else
        {
            // Friendly ships always treat interactables as solid.
            // TODO confirm if this is the behaviour I actually want.
            return 1;
        }
    }
    else if (cell & CELL_GFX_RANGE)
    {
        // Tile cell is returned as solid.
        return 1;
    }
    else if (unitIndex = containsUnit(x, y, unit))
    {
        // Only player units will damage marine life.
        if (unit->unitClass == PLAYER_UNIT_CLASS && units[unitIndex - 1].unitClass == MARINE_LIFE_UNIT_CLASS)
        {
            // Kill the sea creature.
            units[unitIndex - 1].health = 0;

            // Player loses money if they kill marine life.
            gameMoney -= MARINE_LIFE_MONEY_DEDUCTION;

            // Update objectives, potentially ending the game.
            ++turtlesKilled;

            // Return 0 so you can move to that position that the unit was in.
            return 0;
        }

        // Ship or other unit in this square so returned as solid.
        return 1;
    }
    else
    {
        // Everything else would be an empty cell.
        return 0;
    }
}

u8 unitNextPositionIsSolid(Unit* unit, u8 nextPosition)
{
    u8 i = 0;
    u8 j = 0;

    // Checks if every cell that the ship is on is solid or not.
    for (i = 0; i < unit->width; ++i)
    {
        for (j = 0; j < unit->height; ++j)
        {
            // nextPosition determinies if next(x,y) or (x,y) is used.
            u8 x = nextPosition ? unit->nextPosition.x : unit->position.x;
            u8 y = nextPosition ? unit->nextPosition.y : unit->position.y;

            // Check if the current cell is solid.
            if (processCellUnit(unit, x + i, y + j))
            {
                // If one cell is solid then exit the entire loop as we don't need to contiue.
                return TRUE;
            }
        }
    }

    return FALSE;
}

u8 updateMinesAroundUnit(u8 x, u8 y, u8 unitWidth, u8 unitHeight, u8 radius, u8 mineTypeToFind, u8 mineTypeToSet)
{
    // Determine the radius around the radar ship to search.
    u8 searchRadiusStartX = x - radius < 240 ? x - radius : 0;
    u8 searchRadiusStartY = y - radius < 240 ? y - radius : 0;
    u8 searchRadiusEndX = x + unitWidth + radius < MAP_TILES_X ? x + unitWidth + radius : MAP_TILES_X - 1;
    u8 searchRadiusEndY = y + unitHeight + radius < MAP_TILES_Y ? y + unitHeight + radius : MAP_TILES_Y - 1;
    u8 updatedMineCount = 0;

    for (x = searchRadiusStartX; x <= searchRadiusEndX; ++x)
    {
        for (y = searchRadiusStartY; y <= searchRadiusEndY; ++y)
        {
            // Look for a mine at the position within the radius.
            u16 mine = findMine(x, y);

            if (mine)
            {
                // Check if the mine matches the expected mine type.
                if (mapMines[mine] == mineTypeToFind)
                {
                    // Change the mine type to the one to set.
                    // This is either to reveal a bottom mine on initial use 
                    // or to hide a mine after the radar times out.
                    mapMines[mine] = mineTypeToSet;

                    // Increases the number of mines that were updated.
                    ++updatedMineCount;
                }
            }
        }
    }

    // Returning the number of mines updated for scoring purposes
    return updatedMineCount;
}